;;; cl-scopes/test/test-forge

;;;; testing facility for scopes/forge

(defpackage :scopes/test-forge
  (:use :common-lisp)
  (:local-nicknames (:forge :scopes/forge) 
                    (:t :scopes/testing))
  (:export #:run)
  (:import-from :scopes/testing #:deftest #:==))

(in-package :scopes/test-forge)

(defun run ()
  (let ((t:*test-suite* (t:test-suite "forge")))
    (test-seq)
    (test-exec)
    ;(format t "~%data-stack ~a" (dstack))
    (test-def)
    (test-exec-str)
    (test-const)
    (test-var)
    (t:show-result)))

(deftest test-seq ()
  (let ((seq (forge:make-seq)))
    (forge:seq-add seq 1)
    (forge:seq-add seq 2)
    (== (forge:seq-next seq) 1)
    (== (forge:seq-end seq) 2)))

(deftest test-exec ()
  (forge:exec '(4 2 +))
  (== (car (forge:dstack)) 6))

(deftest test-def ()
  (forge:exec '((dup *) "square" def))
  (forge:exec '(7 square))
  (== (car (forge:dstack)) 49))

(deftest test-exec-str ()
  (forge:exec-str "16 square")
  (== (car (forge:dstack)) 256))

(deftest test-const ()
  (forge:exec-str "17 \"c1\" const")
  (forge:exec-str "c1 square")
  (== (car (forge:dstack)) 289))

(deftest test-var ()
  (forge:exec '(24 "v1" var))
  (forge:exec '(v1 get 2 *))
  (== (car (forge:dstack)) 48)
  (forge:exec '(5 v1 put))
  (forge:exec '(v1 get 2 *))
  (== (car (forge:dstack)) 10))
